@@ -1,6 +1,6 @@
 """fontTools.ttLib -- a package for dealing with TrueType fonts.
 
-This package offers translators to convert TrueType fonts to Python 
+This package offers translators to convert TrueType fonts to Python
 objects and vice versa, and additionally from Python to TTX (an XML-based
 text format) and vice versa.
 
@@ -37,7 +37,7 @@ Dumping 'prep' table...
 >>> tt2.importXML("afont.ttx")
 >>> tt2['maxp'].numGlyphs
 242
->>> 
+>>>
 
 """
 
@@ -49,48 +49,50 @@ import sys
 haveMacSupport = 0
 if sys.platform == "mac":
 	haveMacSupport = 1
-elif sys.platform == "darwin" and sys.version_info[:3] != (2, 2, 0):
-	# Python 2.2's Mac support is broken, so don't enable it there.
-	haveMacSupport = 1
+elif sys.platform == "darwin":
+	if sys.version_info[:3] != (2, 2, 0) and sys.version_info[:1] < (3,):
+		# Python 2.2's Mac support is broken, so don't enable it there.
+		# Python 3 does not have Res used by macUtils
+		haveMacSupport = 1
 
 
 class TTLibError(Exception): pass
 
 
 class TTFont(object):
-	
+
 	"""The main font object. It manages file input and output, and offers
-	a convenient way of accessing tables. 
+	a convenient way of accessing tables.
 	Tables will be only decompiled when necessary, ie. when they're actually
 	accessed. This means that simple operations can be extremely fast.
 	"""
-	
+
 	def __init__(self, file=None, res_name_or_index=None,
 			sfntVersion="\000\001\000\000", flavor=None, checkChecksums=False,
 			verbose=False, recalcBBoxes=True, allowVID=False, ignoreDecompileErrors=False,
 			recalcTimestamp=True, fontNumber=-1, lazy=None, quiet=False):
-		
+
 		"""The constructor can be called with a few different arguments.
 		When reading a font from disk, 'file' should be either a pathname
-		pointing to a file, or a readable file object. 
-		
-		It we're running on a Macintosh, 'res_name_or_index' maybe an sfnt 
-		resource name or an sfnt resource index number or zero. The latter 
-		case will cause TTLib to autodetect whether the file is a flat file 
+		pointing to a file, or a readable file object.
+
+		It we're running on a Macintosh, 'res_name_or_index' maybe an sfnt
+		resource name or an sfnt resource index number or zero. The latter
+		case will cause TTLib to autodetect whether the file is a flat file
 		or a suitcase. (If it's a suitcase, only the first 'sfnt' resource
 		will be read!)
-		
+
 		The 'checkChecksums' argument is used to specify how sfnt
 		checksums are treated upon reading a file from disk:
 			0: don't check (default)
 			1: check, print warnings if a wrong checksum is found
 			2: check, raise an exception if a wrong checksum is found.
-		
-		The TTFont constructor can also be called without a 'file' 
-		argument: this is the way to create a new empty font. 
+
+		The TTFont constructor can also be called without a 'file'
+		argument: this is the way to create a new empty font.
 		In this case you can optionally supply the 'sfntVersion' argument,
 		and a 'flavor' which can be None, or 'woff'.
-		
+
 		If the recalcBBoxes argument is false, a number of things will *not*
 		be recalculated upon save/compile:
 			1) glyph bounding boxes
@@ -98,8 +100,8 @@ class TTFont(object):
 			3) hhea min/max values
 		(1) is needed for certain kinds of CJK fonts (ask Werner Lemberg ;-).
 		Additionally, upon importing an TTX file, this option cause glyphs
-		to be compiled right away. This should reduce memory consumption 
-		greatly, and therefore should have some impact on the time needed 
+		to be compiled right away. This should reduce memory consumption
+		greatly, and therefore should have some impact on the time needed
 		to parse/compile large fonts.
 
 		If the recalcTimestamp argument is false, the modified timestamp in the
@@ -124,7 +126,7 @@ class TTFont(object):
 		access only.  If it is set to False, many data structures are loaded
 		immediately.  The default is lazy=None which is somewhere in between.
 		"""
-		
+
 		from fontTools.ttLib import sfnt
 		self.verbose = verbose
 		self.quiet = quiet
@@ -147,6 +149,7 @@ class TTFont(object):
 			self.flavorData = None
 			return
 		if not hasattr(file, "read"):
+			closeStream = True
 			# assume file is a string
 			if haveMacSupport and res_name_or_index is not None:
 				# on the mac, we deal with sfnt resources as well as flat files
@@ -161,25 +164,34 @@ class TTFont(object):
 					file = macUtils.SFNTResourceReader(file, res_name_or_index)
 			else:
 				file = open(file, "rb")
+
 		else:
-			pass # assume "file" is a readable file object
-		self.reader = sfnt.SFNTReader(file, checkChecksums, fontNumber=fontNumber)
+			# assume "file" is a readable file object
+			closeStream = False
+		# read input file in memory and wrap a stream around it to allow overwriting
+		tmp = BytesIO(file.read())
+		if hasattr(file, 'name'):
+			# save reference to input file name
+			tmp.name = file.name
+		if closeStream:
+			file.close()
+		self.reader = sfnt.SFNTReader(tmp, checkChecksums, fontNumber=fontNumber)
 		self.sfntVersion = self.reader.sfntVersion
 		self.flavor = self.reader.flavor
 		self.flavorData = self.reader.flavorData
-	
+
 	def close(self):
 		"""If we still have a reader object, close it."""
 		if self.reader is not None:
 			self.reader.close()
-	
+
 	def save(self, file, makeSuitcase=False, reorderTables=True):
-		"""Save the font to disk. Similarly to the constructor, 
+		"""Save the font to disk. Similarly to the constructor,
 		the 'file' argument can be either a pathname or a writable
 		file object.
-		
+
 		On the Mac, if makeSuitcase is true, a suitcase (resource fork)
-		file will we made instead of a flat .ttf file. 
+		file will we made instead of a flat .ttf file.
 		"""
 		from fontTools.ttLib import sfnt
 		if not hasattr(file, "write"):
@@ -195,33 +207,44 @@ class TTFont(object):
 		else:
 			# assume "file" is a writable file object
 			closeStream = 0
-		
+
 		tags = list(self.keys())
 		if "GlyphOrder" in tags:
 			tags.remove("GlyphOrder")
 		numTables = len(tags)
-		if reorderTables:
-			import tempfile
-			tmp = tempfile.TemporaryFile(prefix="ttx-fonttools")
-		else:
-			tmp = file
+		# write to a temporary stream to allow saving to unseekable streams
+		tmp = BytesIO()
 		writer = sfnt.SFNTWriter(tmp, numTables, self.sfntVersion, self.flavor, self.flavorData)
-		
+
 		done = []
 		for tag in tags:
 			self._writeTable(tag, writer, done)
-		
+
 		writer.close()
 
-		if reorderTables:
+		if (reorderTables is None or writer.reordersTables() or
+				(reorderTables is False and self.reader is None)):
+			# don't reorder tables and save as is
+			file.write(tmp.getvalue())
+			tmp.close()
+		else:
+			if reorderTables is False:
+				# sort tables using the original font's order
+				tableOrder = list(self.reader.keys())
+			else:
+				# use the recommended order from the OpenType specification
+				tableOrder = None
 			tmp.flush()
 			tmp.seek(0)
-			reorderFontTables(tmp, file)
+			tmp2 = BytesIO()
+			reorderFontTables(tmp, tmp2, tableOrder)
+			file.write(tmp2.getvalue())
 			tmp.close()
+			tmp2.close()
 
 		if closeStream:
 			file.close()
-	
+
 	def saveXML(self, fileOrPath, progress=None, quiet=False,
 			tables=None, skipTables=None, splitTables=False, disassembleInstructions=True,
 			bitmapGlyphDataFormat='raw'):
@@ -234,7 +257,7 @@ class TTFont(object):
 		"""
 		from fontTools import version
 		from fontTools.misc import xmlWriter
-		
+
 		self.disassembleInstructions = disassembleInstructions
 		self.bitmapGlyphDataFormat = bitmapGlyphDataFormat
 		if not tables:
@@ -251,19 +274,19 @@ class TTFont(object):
 			idlefunc = getattr(progress, "idle", None)
 		else:
 			idlefunc = None
-		
+
 		writer = xmlWriter.XMLWriter(fileOrPath, idlefunc=idlefunc)
-		writer.begintag("ttFont", sfntVersion=repr(self.sfntVersion)[1:-1], 
+		writer.begintag("ttFont", sfntVersion=repr(self.sfntVersion)[1:-1],
 				ttLibVersion=version)
 		writer.newline()
-		
+
 		if not splitTables:
 			writer.newline()
 		else:
 			# 'fileOrPath' must now be a path
 			path, ext = os.path.splitext(fileOrPath)
 			fileNameTemplate = path + ".%s" + ext
-		
+
 		for i in range(numTables):
 			if progress:
 				progress.set(i)
@@ -290,7 +313,7 @@ class TTFont(object):
 		writer.close()
 		if self.verbose:
 			debugmsg("Done dumping TTX")
-	
+
 	def _tableToXML(self, writer, tag, progress, quiet):
 		if tag in self:
 			table = self[tag]
@@ -322,7 +345,7 @@ class TTFont(object):
 		writer.endtag(xmlTag)
 		writer.newline()
 		writer.newline()
-	
+
 	def importXML(self, file, progress=None, quiet=False):
 		"""Import a TTX file (an XML-based text format), so as to recreate
 		a font object.
@@ -338,12 +361,12 @@ class TTFont(object):
 
 		reader = xmlReader.XMLReader(file, self, progress, quiet)
 		reader.read()
-	
+
 	def isLoaded(self, tag):
-		"""Return true if the table identified by 'tag' has been 
+		"""Return true if the table identified by 'tag' has been
 		decompiled and loaded into memory."""
 		return tag in self.tables
-	
+
 	def has_key(self, tag):
 		if self.isLoaded(tag):
 			return True
@@ -353,9 +376,9 @@ class TTFont(object):
 			return True
 		else:
 			return False
-	
+
 	__contains__ = has_key
-	
+
 	def keys(self):
 		keys = list(self.tables.keys())
 		if self.reader:
@@ -367,10 +390,10 @@ class TTFont(object):
 			keys.remove("GlyphOrder")
 		keys = sortedTagList(keys)
 		return ["GlyphOrder"] + keys
-	
+
 	def __len__(self):
 		return len(list(self.keys()))
-	
+
 	def __getitem__(self, tag):
 		tag = Tag(tag)
 		try:
@@ -407,10 +430,10 @@ class TTFont(object):
 				return table
 			else:
 				raise KeyError("'%s' table not found" % tag)
-	
+
 	def __setitem__(self, tag, table):
 		self.tables[Tag(tag)] = table
-	
+
 	def __delitem__(self, tag):
 		if tag not in self:
 			raise KeyError("'%s' table not found" % tag)
@@ -424,10 +447,10 @@ class TTFont(object):
 			return self[tag]
 		except KeyError:
 			return default
-	
+
 	def setGlyphOrder(self, glyphOrder):
 		self.glyphOrder = glyphOrder
-	
+
 	def getGlyphOrder(self):
 		try:
 			return self.glyphOrder
@@ -442,7 +465,7 @@ class TTFont(object):
 			if glyphOrder is None:
 				#
 				# No names found in the 'post' table.
-				# Try to create glyph names from the unicode cmap (if available) 
+				# Try to create glyph names from the unicode cmap (if available)
 				# in combination with the Adobe Glyph List (AGL).
 				#
 				self._getGlyphNamesFromCmap()
@@ -451,7 +474,7 @@ class TTFont(object):
 		else:
 			self._getGlyphNamesFromCmap()
 		return self.glyphOrder
-	
+
 	def _getGlyphNamesFromCmap(self):
 		#
 		# This is rather convoluted, but then again, it's an interesting problem:
@@ -489,7 +512,10 @@ class TTFont(object):
 		# to work with (so we don't get called recursively).
 		self.glyphOrder = glyphOrder
 		# Get a (new) temporary cmap (based on the just invented names)
-		tempcmap = self['cmap'].getcmap(3, 1)
+		try:
+			tempcmap = self['cmap'].getcmap(3, 1)
+		except KeyError:
+			tempcmap = None
 		if tempcmap is not None:
 			# we have a unicode cmap
 			from fontTools import agl
@@ -525,19 +551,19 @@ class TTFont(object):
 			# restore partially loaded cmap, so it can continue loading
 			# using the proper names.
 			self.tables['cmap'] = cmapLoading
-	
+
 	def getGlyphNames(self):
 		"""Get a list of glyph names, sorted alphabetically."""
 		glyphNames = sorted(self.getGlyphOrder()[:])
 		return glyphNames
-	
+
 	def getGlyphNames2(self):
-		"""Get a list of glyph names, sorted alphabetically, 
+		"""Get a list of glyph names, sorted alphabetically,
 		but not case sensitive.
 		"""
 		from fontTools.misc import textTools
 		return textTools.caselessSort(self.getGlyphOrder())
-	
+
 	def getGlyphName(self, glyphID, requireReal=False):
 		try:
 			return self.getGlyphOrder()[glyphID]
@@ -547,7 +573,7 @@ class TTFont(object):
 				# the cmap table than there are glyphs. I don't think it's legal...
 				return "glyph%.5d" % glyphID
 			else:
-				# user intends virtual GID support 	
+				# user intends virtual GID support
 				try:
 					glyphName = self.VIDDict[glyphID]
 				except KeyError:
@@ -577,7 +603,7 @@ class TTFont(object):
 						except (NameError, ValueError):
 							raise KeyError(glyphName)
 				else:
-					# user intends virtual GID support 	
+					# user intends virtual GID support
 					try:
 						glyphID = self.reverseVIDDict[glyphName]
 					except KeyError:
@@ -610,9 +636,9 @@ class TTFont(object):
 		glyphOrder = self.getGlyphOrder()
 		for glyphID in range(len(glyphOrder)):
 			d[glyphOrder[glyphID]] = glyphID
-	
+
 	def _writeTable(self, tag, writer, done):
-		"""Internal helper function for self.save(). Keeps track of 
+		"""Internal helper function for self.save(). Keeps track of
 		inter-table dependencies.
 		"""
 		if tag in done:
@@ -629,7 +655,7 @@ class TTFont(object):
 			debugmsg("writing '%s' table to disk" % tag)
 		writer[tag] = tabledata
 		done.append(tag)
-	
+
 	def getTableData(self, tag):
 		"""Returns raw table data, whether compiled or directly read from disk.
 		"""
@@ -644,51 +670,52 @@ class TTFont(object):
 			return self.reader[tag]
 		else:
 			raise KeyError(tag)
-	
+
 	def getGlyphSet(self, preferCFF=True):
 		"""Return a generic GlyphSet, which is a dict-like object
 		mapping glyph names to glyph objects. The returned glyph objects
 		have a .draw() method that supports the Pen protocol, and will
-		have an attribute named 'width', but only *after* the .draw() method
-		has been called.
-		
+		have an attribute named 'width'.
+
 		If the font is CFF-based, the outlines will be taken from the 'CFF '
 		table. Otherwise the outlines will be taken from the 'glyf' table.
 		If the font contains both a 'CFF ' and a 'glyf' table, you can use
 		the 'preferCFF' argument to specify which one should be taken.
 		"""
-		if preferCFF and "CFF " in self:
-			return list(self["CFF "].cff.values())[0].CharStrings
-		if "glyf" in self:
-			return _TTGlyphSet(self)
-		if "CFF " in self:
-			return list(self["CFF "].cff.values())[0].CharStrings
-		raise TTLibError("Font contains no outlines")
+		glyphs = None
+		if (preferCFF and "CFF " in self) or "glyf" not in self:
+			glyphs = _TTGlyphSet(self, list(self["CFF "].cff.values())[0].CharStrings, _TTGlyphCFF)
+
+		if glyphs is None and "glyf" in self:
+			glyphs = _TTGlyphSet(self, self["glyf"], _TTGlyphGlyf)
+
+		if glyphs is None:
+			raise TTLibError("Font contains no outlines")
+
+		return glyphs
 
 
 class _TTGlyphSet(object):
-	
-	"""Generic dict-like GlyphSet class, meant as a TrueType counterpart
-	to CFF's CharString dict. See TTFont.getGlyphSet().
+
+	"""Generic dict-like GlyphSet class that pulls metrics from hmtx and
+	glyph shape from TrueType or CFF.
 	"""
-	
-	# This class is distinct from the 'glyf' table itself because we need
-	# access to the 'hmtx' table, which could cause a dependency problem
-	# there when reading from XML.
-	
-	def __init__(self, ttFont):
-		self._ttFont = ttFont
-	
+
+	def __init__(self, ttFont, glyphs, glyphType):
+		self._glyphs = glyphs
+		self._hmtx = ttFont['hmtx']
+		self._glyphType = glyphType
+
 	def keys(self):
-		return list(self._ttFont["glyf"].keys())
-	
+		return list(self._glyphs.keys())
+
 	def has_key(self, glyphName):
-		return glyphName in self._ttFont["glyf"]
-	
+		return glyphName in self._glyphs
+
 	__contains__ = has_key
 
 	def __getitem__(self, glyphName):
-		return _TTGlyph(glyphName, self._ttFont)
+		return self._glyphType(self, self._glyphs[glyphName], self._hmtx[glyphName])
 
 	def get(self, glyphName, default=None):
 		try:
@@ -696,77 +723,48 @@ class _TTGlyphSet(object):
 		except KeyError:
 			return default
 
-
 class _TTGlyph(object):
-	
+
 	"""Wrapper for a TrueType glyph that supports the Pen protocol, meaning
 	that it has a .draw() method that takes a pen object as its only
 	argument. Additionally there is a 'width' attribute.
 	"""
-	
-	def __init__(self, glyphName, ttFont):
-		self._glyphName = glyphName
-		self._ttFont = ttFont
-		self.width, self.lsb = self._ttFont['hmtx'][self._glyphName]
-	
+
+	def __init__(self, glyphset, glyph, metrics):
+		self._glyphset = glyphset
+		self._glyph = glyph
+		self.width, self.lsb = metrics
+
 	def draw(self, pen):
 		"""Draw the glyph onto Pen. See fontTools.pens.basePen for details
 		how that works.
 		"""
-		glyfTable = self._ttFont['glyf']
-		glyph = glyfTable[self._glyphName]
-		if hasattr(glyph, "xMin"):
-			offset = self.lsb - glyph.xMin
-		else:
-			offset = 0
-		if glyph.isComposite():
-			for component in glyph:
-				glyphName, transform = component.getComponentInfo()
-				pen.addComponent(glyphName, transform)
-		else:
-			coordinates, endPts, flags = glyph.getCoordinates(glyfTable)
-			if offset:
-				coordinates = coordinates + (offset, 0)
-			start = 0
-			for end in endPts:
-				end = end + 1
-				contour = coordinates[start:end]
-				cFlags = flags[start:end]
-				start = end
-				if 1 not in cFlags:
-					# There is not a single on-curve point on the curve,
-					# use pen.qCurveTo's special case by specifying None
-					# as the on-curve point.
-					contour.append(None)
-					pen.qCurveTo(*contour)
-				else:
-					# Shuffle the points so that contour the is guaranteed
-					# to *end* in an on-curve point, which we'll use for
-					# the moveTo.
-					firstOnCurve = cFlags.index(1) + 1
-					contour = contour[firstOnCurve:] + contour[:firstOnCurve]
-					cFlags = cFlags[firstOnCurve:] + cFlags[:firstOnCurve]
-					pen.moveTo(contour[-1])
-					while contour:
-						nextOnCurve = cFlags.index(1) + 1
-						if nextOnCurve == 1:
-							pen.lineTo(contour[0])
-						else:
-							pen.qCurveTo(*contour[:nextOnCurve])
-						contour = contour[nextOnCurve:]
-						cFlags = cFlags[nextOnCurve:]
-				pen.closePath()
+		self._glyph.draw(pen)
+
+class _TTGlyphCFF(_TTGlyph):
+	pass
+
+class _TTGlyphGlyf(_TTGlyph):
+
+	def draw(self, pen):
+		"""Draw the glyph onto Pen. See fontTools.pens.basePen for details
+		how that works.
+		"""
+		glyfTable = self._glyphset._glyphs
+		glyph = self._glyph
+		offset = self.lsb - glyph.xMin if hasattr(glyph, "xMin") else 0
+		glyph.draw(pen, glyfTable, offset)
 
 
 class GlyphOrder(object):
-	
+
 	"""A pseudo table. The glyph order isn't in the font as a separate
 	table, but it's nice to present it as such in the TTX format.
 	"""
-	
+
 	def __init__(self, tag=None):
 		pass
-	
+
 	def toXML(self, writer, ttFont):
 		glyphOrder = ttFont.getGlyphOrder()
 		writer.comment("The 'id' attribute is only for humans; "
@@ -776,7 +774,7 @@ class GlyphOrder(object):
 			glyphName = glyphOrder[i]
 			writer.simpletag("GlyphID", id=i, name=glyphName)
 			writer.newline()
-	
+
 	def fromXML(self, name, attrs, content, ttFont):
 		if not hasattr(self, "glyphOrder"):
 			self.glyphOrder = []
@@ -786,7 +784,7 @@ class GlyphOrder(object):
 
 
 def getTableModule(tag):
-	"""Fetch the packer/unpacker module for a table. 
+	"""Fetch the packer/unpacker module for a table.
 	Return None when no module is found.
 	"""
 	from . import tables
@@ -807,7 +805,7 @@ def getTableModule(tag):
 
 
 def getTableClass(tag):
-	"""Fetch the packer/unpacker class for a table. 
+	"""Fetch the packer/unpacker class for a table.
 	Return None when no class is found.
 	"""
 	module = getTableModule(tag)
@@ -827,7 +825,6 @@ def getClassTag(klass):
 	return identifierToTag(name)
 
 
-
 def newTable(tag):
 	"""Return a new instance of a table."""
 	tableClass = getTableClass(tag)
@@ -846,14 +843,14 @@ def _escapechar(c):
 
 
 def tagToIdentifier(tag):
-	"""Convert a table tag to a valid (but UGLY) python identifier, 
-	as well as a filename that's guaranteed to be unique even on a 
+	"""Convert a table tag to a valid (but UGLY) python identifier,
+	as well as a filename that's guaranteed to be unique even on a
 	caseless file system. Each character is mapped to two characters.
 	Lowercase letters get an underscore before the letter, uppercase
 	letters get an underscore after the letter. Trailing spaces are
 	trimmed. Illegal characters are escaped as two hex bytes. If the
 	result starts with a number (as the result of a hex escape), an
-	extra underscore is prepended. Examples: 
+	extra underscore is prepended. Examples:
 		'glyf' -> '_g_l_y_f'
 		'cvt ' -> '_c_v_t'
 		'OS/2' -> 'O_S_2f_2'
@@ -928,11 +925,11 @@ def debugmsg(msg):
 
 # Table order as recommended in the OpenType specification 1.4
 TTFTableOrder = ["head", "hhea", "maxp", "OS/2", "hmtx", "LTSH", "VDMX",
-                  "hdmx", "cmap", "fpgm", "prep", "cvt ", "loca", "glyf",
-                  "kern", "name", "post", "gasp", "PCLT"]
+				"hdmx", "cmap", "fpgm", "prep", "cvt ", "loca", "glyf",
+				"kern", "name", "post", "gasp", "PCLT"]
 
 OTFTableOrder = ["head", "hhea", "maxp", "OS/2", "name", "cmap", "post",
-                  "CFF "]
+				"CFF "]
 
 def sortedTagList(tagList, tableOrder=None):
 	"""Return a sorted copy of tagList, sorted according to the OpenType
@@ -982,10 +979,11 @@ def maxPowerOfTwo(x):
 	return max(exponent - 1, 0)
 
 
-def getSearchRange(n, itemSize):
+def getSearchRange(n, itemSize=16):
 	"""Calculate searchRange, entrySelector, rangeShift.
 	"""
-	# This stuff needs to be stored in the file, because?
+	# itemSize defaults to 16, for backward compatibility
+	# with upstream fonttools.
 	exponent = maxPowerOfTwo(n)
 	searchRange = (2 ** exponent) * itemSize
 	entrySelector = exponent
